@@ -14,13 +14,15 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import functools
 from types import SimpleNamespace
+from typing import Optional
 
 import torch
 
 from .jit import JitSpec
 from .jit import gen_act_and_mul_module as gen_act_and_mul_module_impl
-from .utils import register_custom_op, register_fake_op
+from .utils import device_support_pdl, register_custom_op, register_fake_op
 
 silu_def_cu_str = r"""
 __device__ __forceinline__ float silu(const float& val) {
@@ -50,52 +52,48 @@ act_func_def_str = {
 }
 
 
-_jit_modules = {}
-
-
 def gen_act_and_mul_module(act_func_name: str) -> JitSpec:
     return gen_act_and_mul_module_impl(act_func_name, act_func_def_str[act_func_name])
 
 
+@functools.cache
 def get_act_and_mul_module(act_func_name: str):
-    global _jit_modules
-    if act_func_name not in _jit_modules:
-        module = gen_act_and_mul_module(act_func_name).build_and_load()
+    module = gen_act_and_mul_module(act_func_name).build_and_load()
 
-        # torch library for act_and_mul
-        fname = f"{act_func_name}_and_mul"
-        fn = getattr(module, fname).default
+    # torch library for act_and_mul
+    fname = f"{act_func_name}_and_mul"
+    fn = getattr(module, fname).default
 
-        @register_custom_op(f"flashinfer::{fname}", mutates_args=("out",))
-        def _act_and_mul(
-            out: torch.Tensor, input: torch.Tensor, enable_pdl: bool = False
-        ) -> None:
-            fn(out, input, enable_pdl)
+    @register_custom_op(f"flashinfer::{fname}", mutates_args=("out",))
+    def _act_and_mul(
+        out: torch.Tensor, input: torch.Tensor, enable_pdl: Optional[bool] = None
+    ) -> None:
+        if enable_pdl is None:
+            enable_pdl = device_support_pdl(input.device)
+        fn(out, input, enable_pdl)
 
-        @register_fake_op(f"flashinfer::{fname}")
-        def _fake_act_and_mul(
-            out: torch.Tensor, input: torch.Tensor, enable_pdl: bool = False
-        ) -> None:
-            pass
+    @register_fake_op(f"flashinfer::{fname}")
+    def _fake_act_and_mul(
+        out: torch.Tensor, input: torch.Tensor, enable_pdl: Optional[bool] = None
+    ) -> None:
+        pass
 
-        # Register the module
-        _jit_modules[act_func_name] = SimpleNamespace(**{fname: _act_and_mul})
-
-    return _jit_modules[act_func_name]
+    # Register the module
+    return SimpleNamespace(**{fname: _act_and_mul})
 
 
 def _check_shape(input: torch.Tensor, output: torch.Tensor) -> None:
     assert input.ndim == output.ndim, f"{input.ndim} != {output.ndim}"
-    assert (
-        input.shape[:-1] == output.shape[:-1]
-    ), f"{input.shape[:-1]} != {output.shape[:-1]}"
-    assert (
-        input.shape[-1] == 2 * output.shape[-1]
-    ), f"{input.shape[-1]} != {2 * output.shape[-1]}"
+    assert input.shape[:-1] == output.shape[:-1], (
+        f"{input.shape[:-1]} != {output.shape[:-1]}"
+    )
+    assert input.shape[-1] == 2 * output.shape[-1], (
+        f"{input.shape[-1]} != {2 * output.shape[-1]}"
+    )
 
 
 def silu_and_mul(
-    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: bool = False
+    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: Optional[bool] = None
 ) -> torch.Tensor:
     r"""Fused SiLU and Mul operation.
 
@@ -118,6 +116,8 @@ def silu_and_mul(
     output: torch.Tensor
         Output tensor, shape (..., hidden_size).
     """
+    if enable_pdl is None:
+        enable_pdl = device_support_pdl(input.device)
     if input.shape[-1] * input.dtype.itemsize % 16 != 0:
         raise ValueError("The pointers must be multiple of 16 bytes.")
     if out is not None:
@@ -137,7 +137,7 @@ def silu_and_mul(
 
 
 def gelu_tanh_and_mul(
-    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: bool = False
+    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: Optional[bool] = None
 ) -> torch.Tensor:
     r"""Fused GeLU Tanh and Mul operation.
 
@@ -160,6 +160,8 @@ def gelu_tanh_and_mul(
     output: torch.Tensor
         Output tensor, shape (..., hidden_size).
     """
+    if enable_pdl is None:
+        enable_pdl = device_support_pdl(input.device)
     if input.shape[-1] * input.dtype.itemsize % 16 != 0:
         raise ValueError("The pointers must be multiple of 16 bytes.")
     if out is not None:
@@ -175,7 +177,7 @@ def gelu_tanh_and_mul(
 
 
 def gelu_and_mul(
-    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: bool = False
+    input: torch.Tensor, out: torch.Tensor = None, enable_pdl: Optional[bool] = None
 ) -> torch.Tensor:
     r"""Fused GeLU and Mul operation.
 
@@ -198,6 +200,8 @@ def gelu_and_mul(
     output: torch.Tensor
         Output tensor, shape (..., hidden_size).
     """
+    if enable_pdl is None:
+        enable_pdl = device_support_pdl(input.device)
     if input.shape[-1] * input.dtype.itemsize % 16 != 0:
         raise ValueError("The pointers must be multiple of 16 bytes.")
     if out is not None:
