@@ -22,7 +22,7 @@ import torch
 from .jit import JitSpec
 from .jit import env as jit_env
 from .jit import gen_batch_mla_module, gen_jit_spec, sm100a_nvcc_flags
-from .utils import MaskMode, _check_shape_dtype_device, determine_mla_backend
+from .utils import MaskMode, check_shape_dtype_device, determine_mla_backend
 
 
 def _check_cutlass_shape(q_nope_pe, ckv_kpe_cache, kv_len, page_table):
@@ -394,7 +394,7 @@ class BatchMLAPagedAttentionWrapper:
             if out is None:
                 out = torch.empty_like(q_nope)
             else:
-                _check_shape_dtype_device(
+                check_shape_dtype_device(
                     out, q_nope.shape, q_nope.dtype, q_nope.device, "out"
                 )
             q_nope_pe = torch.cat([q_nope, q_pe], dim=-1)
@@ -426,7 +426,7 @@ class BatchMLAPagedAttentionWrapper:
         if out is None:
             out = torch.empty_like(q_nope)
         else:
-            _check_shape_dtype_device(
+            check_shape_dtype_device(
                 out, q_nope.shape, q_nope.dtype, q_nope.device, "out"
             )
 
@@ -434,7 +434,7 @@ class BatchMLAPagedAttentionWrapper:
             if lse is None:
                 lse = torch.empty(q_nope.shape[:2], dtype=torch.float32, device=device)
             else:
-                _check_shape_dtype_device(
+                check_shape_dtype_device(
                     lse, q_nope.shape[:2], torch.float32, q_nope.device, "lse"
                 )
         profiler_args = (profiler_buffer,) if self._use_profiler else ()
