@@ -62,7 +62,12 @@ enum class ActType {
   // beta' = beta / scaleAb, scaleC' = scaleC * scaleAb.
   //
   // GatedSilu is a special case of SwiGlu where the alpha is 1.0 and the beta is 0.0.
-  SwiGlu
+  SwiGlu,
+  // For ActType == GeGlu, we use the simplified version
+  //    gatedAct = scaleC' * (x0 + beta') * ((x1 * scaleGate) * phi(alpha * x1 * scaleGate)),
+  // where x0 and x1 are the raw numbers from Gemm, while scaleC and scaleGate are input scales,
+  // beta' = beta / scaleAb, scaleC' = scaleC * scaleAb.
+  GeGlu,
 };
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -73,6 +78,7 @@ enum class ActType {
   inline bool is##actType(ActType type) { return (type == ActType::actType); }
 
 TLLM_ACT_TYPE_FUNCTION(SwiGlu)
+TLLM_ACT_TYPE_FUNCTION(GeGlu)
 
 #undef TLLM_ACT_TYPE_FUNCTION
 
@@ -82,6 +88,8 @@ inline std::string getActTypeName(ActType type) {
   switch (type) {
     case ActType::SwiGlu:
       return "SwiGlu";
+    case ActType::GeGlu:
+      return "GeGlu";
     default:
       return "Unknown type";
   }
@@ -158,8 +166,8 @@ inline bool checkAndUpdateGemmGatedActOptions(gemmGatedAct::GemmGatedActOptions&
 inline std::string dumpOptions(GemmGatedActOptions const& options) {
   std::stringstream ss;
   ss << gemm::dumpOptions(options) << ", ";
-  ss << "mActType="
-     << "gemmGatedAct::ActType(" << static_cast<int32_t>(options.mActType) << ")," << std::endl;
+  ss << "mActType=" << "gemmGatedAct::ActType(" << static_cast<int32_t>(options.mActType) << "),"
+     << std::endl;
   ss << "mClampBeforeAct=" << options.mClampBeforeAct << "" << std::endl;
   return ss.str();
 }
