@@ -64,9 +64,7 @@ def swizzle_blockscale(
         _pad_scale_factors(unswizzled_sf[i], m, n, sf_vec_size) for i in range(b)
     ]
     padded_input_sf = torch.stack(padded_input_sf_chunked)
-    out = get_fp4_quantization_module().nvfp4_block_scale_interleave_sm100(
-        padded_input_sf
-    )
+    out = get_fp4_quantization_module().block_scale_interleave_sm100(padded_input_sf)
     out = out.view(padded_input_sf.shape)
     return out
 
